#!/usr/bin/env bash
#===============================================================================
#
# DIRECTORY:
#   ---
#
# FILE:
#   ./accounts_plugin.sh
#
# USAGE:
#     . accounts_plugin.sh
#   OR
#     source accounts_plugin.sh
#
# DESCRIPTION:
#   Accounts plugin (library) to generate content with simple template engine.
#
# BUGS:
#   ---
#
# TESTS:
#   - shellcheck -s bash -e SC2034 ./accounts_plugin.sh
#   - shellcheck -s ksh -e SC2034 ./accounts_plugin.sh
#   - shellcheck -s dash -e SC2034 ./accounts_plugin.sh
#
# AUTHOR:
#   Patrick Neumann, patrick@neumannsland.de
#
# COAUTHOR(S):
#   Odin Heitmann, odin.heitmann@gmail.com
#
# COMPANY:
#   (privately)
#
# VERSION:
#   1.0
#
# LINK TO THE MOST CURRENT VERSION:
#   (Sorry, we bet, I'm not allowed to publish it over GitHub!)
#
# CREATED:
#   2018-06-01
#
# COPYRIGHT (C):
#   2018 - Patrick Neumann & Odin Heitmann
#
# LICENSE:
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
# WARRANTY:
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# NOTES:
#   ---
#
# TODO:
#   ---
#
# HISTORY:
#   1.0 - P. N. & O. H. - Initial (for the trainers eyes only) release
#
#===============================================================================

#-------------------------------------------------------------------------------
# Check if functions library is loaded first.
#-------------------------------------------------------------------------------
if [ "${FUNCTIONS_LOADED}" != "true" ] ; then
  printf "\\n\\033[01;31;40mERROR: functions library not loaded... EXIT!!!\\033[00m\\n\\n"
  exit 200
fi

#-------------------------------------------------------------------------------
# Check if temblate engine library is loaded first.
#-------------------------------------------------------------------------------
if [ "${TEMPLATE_ENGINE_LOADED}" != "true" ] ; then
  error_exit "template_engine.sh not loaded" 200
fi

#=== CONFIGURATION (static) ====================================================
#assign_binary "xyz"                                         # (un)common

#=== FUNCTION ==================================================================
# NAME: get_accounts_list
# DESCRIPTION: "Grep" information and store into vars.
# PARAMETERS: none (global vars will be used.)
#===============================================================================
get_accounts_list () {
  # "943624fd13e27b800cc6d9ce1100c22356ee365c" in iTunes Backup.
  readonly ACCOUNTS_SQLITE="$( ${BIN_FIND} "${IOS_BACKUP}" -type f -name "Accounts3.sqlite" )"
	
  if [ -z "${ACCOUNTS_SQLITE}" ] ; then
    readonly ACCOUNTS_SOURCE="<code>Accounts3.sqlite not found</code>"
		error "Accounts3.sqlite not found"
    readonly ACCOUNTS_LIST="<tr><td class='highlight' colspan='4'>ERROR: database not found!</td></tr>"
	else
    readonly ACCOUNTS_SOURCE="<a href='Accounts3.sqlite' target='_blank'><code>${ACCOUNTS_SQLITE/${IOS_BACKUP}/\/private\/var}</code></a>"
    # * -> don't forget to copy write ahead log!
    ${BIN_CP}	"${ACCOUNTS_SQLITE}"* "${FOLDER}/"

	  readonly ACCOUNTS_LIST="$( ${BIN_SQLITE3} -html "${FOLDER}/Accounts3.sqlite" \
	                            "SELECT a.Z_PK, \
	                              DATETIME(a.ZDATE+978307200, 'unixepoch'), \
	                              t.ZACCOUNTTYPEDESCRIPTION, \
	                              a.ZUSERNAME \
	                            FROM ZACCOUNT AS a, \
	                              ZACCOUNTTYPE AS t \
	                            WHERE a.ZACCOUNTTYPE=t.Z_PK;" )"
  fi

  if [ "${DEBUG}" = "on" ] ; then
    ${BIN_PRINTF} "INFO: accounts_plugin processed.\\n" 1>&2
	fi
}

#-------------------------------------------------------------------------------
# Partial template for hardware information for the simple template engine.
#-------------------------------------------------------------------------------
# SC2034: all "unused" vars are verified!
ACCOUNTS_INFORMATION=$( ${BIN_CAT} <<'EOF'
          <h2>Accounts</h2>
          <h4>Sources</h4>
          <ul class="file">
						<li>${ACCOUNTS_SOURCE}</li>
          </ul>
          <h3>Information</h3>
         <table>
            <tr>
              <th>ID</th>
              <th>Date</th>
              <th>Description</th>
              <th>Username</th>
            </tr>
          ${ACCOUNTS_LIST}
					</table>
EOF
)

readonly ACCOUNTS_PLUGIN_LOADED="true"

if [ "${DEBUG}" = "on" ] ; then
  ${BIN_PRINTF} "INFO: accounts_plugin.sh loaded.\\n" 1>&2
fi
# Do not use "exit" at the end of a sourced library!